# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import generators
from __future__ import division

import os

from PySide2 import QtCore, QtWidgets
from PySide2.QtUiTools import QUiLoader

from maya.app.general import mayaMixin

CURRENT_PATH = os.path.dirname(os.path.abspath(__file__))

settings_path = None
if os.name == "nt":
    settings_path = os.environ["APPDATA"]
else:
    settings_path = "library/preferences"


SETTING_CURRENT_PATH = settings_path


class View(mayaMixin.MayaQWidgetDockableMixin, QtWidgets.QMainWindow):
    def __init__(self, *args, **kwargs):
        super(View, self).__init__(*args, **kwargs)
        self.ui = QUiLoader().load(os.path.join(CURRENT_PATH, "ui", "MayaAutoPackageReloader.ui"))
        self.setCentralWidget(self.ui)
        self.setWindowTitle(self.ui.windowTitle())
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)

        self.comp = QtWidgets.QCompleter(self)
        self.comp.setCompletionMode(QtWidgets.QCompleter.UnfilteredPopupCompletion)

        self.settings = QtCore.QSettings(os.path.join(SETTING_CURRENT_PATH, "setting.ini"), QtCore.QSettings.IniFormat)

    def showEvent(self, event):
        super(View, self).showEvent(event)

        self.__load_settings()

    def closeEvent(self, event):
        super(View, self).closeEvent(event)

        self.__save_settings()

    def __load_settings(self):
        self.ui.ReloadTargetBox.setCurrentText(self.settings.value(self.ui.ReloadTargetBox.objectName(), ""))
        self.ui.PathLineEdit.setText(self.settings.value(self.ui.PathLineEdit.objectName(), ""))

    def __save_settings(self):
        self.settings.setValue(self.ui.ReloadTargetBox.objectName(), self.ui.ReloadTargetBox.currentText())
        self.settings.setValue(self.ui.PathLineEdit.objectName(), self.ui.PathLineEdit.text())



# -*- coding: utf-8 -*-
"""watchdog server.
"""
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import generators
from __future__ import division
import logging

from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

from . import app


class ReloadServer(object):
    def __init__(self):
        self.observer = None
        self.is_monitor = False

    def start(self, target_path, target_module):
        self.observer = Observer()
        self.observer.schedule(MayaModuleReloadEventhandler(target_module), target_path, recursive=True)

        self.observer.start()
        self.is_monitor = True

    def stop(self):
        self.observer.stop()
        self.observer.join()

        self.is_monitor = False


class MayaModuleReloadEventhandler(PatternMatchingEventHandler):
    def __init__(self, reload_target_name):
        super(MayaModuleReloadEventhandler, self).__init__(patterns=["*.py"], ignore_patterns=["*.pyc"])
        self.reload_target_name = reload_target_name

        self.logger = logging.getLogger("mayarepacker")

    def on_any_event(self, event):
        self.logger.info("================================================")
        self.logger.info("reload name : {}".format(self.reload_target_name))
        self.logger.info("event name : {}".format(event.event_type))
        self.logger.info("event path : {}".format(event.src_path))
        self.logger.info("================================================")

        app.remove_include_module(self.reload_target_name)



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import generators
from __future__ import division

import logging

from .controller import main


__version__ = '1.0.0'

logger = logging.getLogger("mayarepacker")
logger.setLevel(logging.DEBUG)
logger.propagate = False



import logging
import sys


def remove_include_module(name):
    logger = logging.getLogger("mayarepacker")

    module_list = [_ for _ in sys.modules]
    module_list.sort()

    logger.info("================================================")

    first_hit_parent_module = None
    for k in module_list:
        if k.startswith(name):
            if first_hit_parent_module is None:
                first_hit_parent_module = k

            del sys.modules[k]
            # send_message_list.append("remove module : {}".format(k))
            logger.info("remove module : {}".format(k))

    if first_hit_parent_module:
        exec("import {}".format(first_hit_parent_module))
        logger.info("reloaded : {}".format(first_hit_parent_module))

    logger.info("================================================")



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import generators
from __future__ import division

import threading
import sys
import logging
from functools import partial

from . import view
from . import server

from PySide2 import QtCore

from maya import cmds


class Controller(object):
    def __init__(self):
        self.gui = view.View()

        self.reload_server = server.ReloadServer()
        self._filter = None

        self.setup_event()

        self.logger = logging.getLogger("mayarepacker")

        self.logger.handlers = []
        self.logger.addHandler(GUILogHandler(self.gui.ui.ConsoleTextEdit))

    def setup_event(self):
        self.gui.ui.OpenFolderButton.clicked.connect(self.select_folder_dialog)
        self.gui.ui.StartMonitorButton.clicked.connect(self.start_monitor)
        self.gui.ui.EndMonitorButton.clicked.connect(self.end_monitor)
        self.gui.ui.ManualReloadButton.clicked.connect(self.reload_manual)

        self.gui.ui.ConsoleTextEdit.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.gui.ui.ConsoleTextEdit.customContextMenuRequested.connect(self.clear_console)

        self.__update_completer()

        self._filter = Filter(self.end_monitor, self.__update_completer)
        self.gui.installEventFilter(self._filter)

    def select_folder_dialog(self):
        select_path = cmds.fileDialog2(dialogStyle=2, fileMode=3)[0]
        if not select_path:
            return

        self.gui.ui.PathLineEdit.setText(select_path)

    def start_monitor(self):
        target_path = self.gui.ui.PathLineEdit.text()
        target_module = self.gui.ui.ReloadTargetBox.currentText()

        if not target_path or not target_module:
            return

        thread_1 = threading.Thread(name="MonitoringThread", target=self.reload_server.start, args=[target_path, target_module])
        thread_1.start()

        self.gui.ui.PathLineEdit.setEnabled(False)
        self.gui.ui.OpenFolderButton.setEnabled(False)
        self.gui.ui.ReloadTargetBox.setEnabled(False)

        self.gui.ui.StartMonitorButton.setEnabled(False)
        self.gui.ui.EndMonitorButton.setEnabled(True)
        self.gui.ui.ManualReloadButton.setEnabled(False)

    def end_monitor(self):
        self.reload_server.stop()

        self.gui.ui.PathLineEdit.setEnabled(True)
        self.gui.ui.OpenFolderButton.setEnabled(True)
        self.gui.ui.ReloadTargetBox.setEnabled(True)

        self.gui.ui.StartMonitorButton.setEnabled(True)
        self.gui.ui.EndMonitorButton.setEnabled(False)
        self.gui.ui.ManualReloadButton.setEnabled(True)

    def __update_completer(self):
        if not self.reload_server.is_monitor:
            current_text = self.gui.ui.ReloadTargetBox.currentText()

            self.gui.ui.ReloadTargetBox.clear()

            target_completer = [_ for _ in sys.modules]
            target_completer.sort()

            self.gui.ui.ReloadTargetBox.addItems(target_completer)
            self.gui.ui.ReloadTargetBox.setCurrentText(current_text)

            self.gui.comp.setModel(QtCore.QStringListModel(target_completer))
            self.gui.ui.ReloadTargetBox.setCompleter(self.gui.comp)

    def reload_manual(self):
        from . import app

        target_module = self.gui.ui.ReloadTargetBox.currentText()
        if not target_module:
            return

        app.remove_include_module(target_module)

    def clear_console(self):
        self.gui.ui.ConsoleTextEdit.clear()


class Filter(QtCore.QObject):
    def __init__(self, close_event, focus_event):
        super(Filter, self).__init__()
        self.__close_event = close_event
        self.__focus_event = focus_event

    def eventFilter(self, widget, event):
        if event.type() == QtCore.QEvent.Close:
            self.__close_event()
        if event.type() == QtCore.QEvent.WindowActivate:
            self.__focus_event()

        return False


def main():
    global MayaAutoReloader

    try:
        MayaAutoReloader.gui.close()
    except:
        pass

    MayaAutoReloader = Controller()
    MayaAutoReloader.gui.show()


class GUILogHandler(logging.Handler):
    def __init__(self, plain_text_edit, level=logging.NOTSET):
        super(GUILogHandler, self).__init__(level=level)
        self.plain_text_edit = plain_text_edit
        self.createLock()

    def emit(self, record):
        import maya
        if len(record.msg) > 0:
            msg = self.format(record)
        else:
            msg = ""

        maya.utils.executeInMainThreadWithResult(partial(self.plain_text_edit.verticalScrollBar().setValue, self.plain_text_edit.verticalScrollBar().maximum()))
        maya.utils.executeInMainThreadWithResult(partial(self.plain_text_edit.appendPlainText, msg))
